/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2015 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technoledge Corp. 
//  Website: http://www.nuvoton.com
//  E-Mail : MicroC-8bit@nuvoton.com
//  Date   : Sep/1/2015
//***********************************************************************************************************

//***********************************************************************************************************
//  File Function: N76E616 UART-0 Mode2 demo code
//***********************************************************************************************************

#include <stdio.h>
#include "N76E616.h"
#include "Version.h"
#include "Typedef.h"
#include "Define.h"
#include "SFR_Macro.h"
#include "Common.h"
#include "Delay.h"

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
//<e0> System Clock Source Configuration
// <o1> System Clock Source Selection
//      <0=> 2~16MHz    XTAL
//      <1=> 32.768KHz  XTAL
//      <2=> 11.0592MHz Internal
//      <3=> 10KHz      Internal
//      <4=> OSC-In     External
//</e>
//
//<e2> Clock Divider Configuration
//     <o3.0..7>  System Clock Source Devider
//                     <i> Fsys = (System Clock Source) / (2 * Devider)
//                     <1-255>
//</e>
//
//-------- <<< end of configuration section >>> ------------------------------
*/

#define SYS_CLK_EN      1
#define SYS_SEL         2
#define SYS_DIV_EN      1                               //0: Fsys=Fosc, 1: Fsys = Fosc/(2*CKDIV)
#define SYS_DIV         9
#define Receive_Buffer_Size 2 // Receive Buffer Size. (0 - 255)
BIT Receive_Get = 0,Transmit_Busy;
UINT8 u8Receive_Buffer_Index = 0;
UINT8 xdata Receive_Buffer[Receive_Buffer_Size] = {0};  // Receive_Buffer.
bit BIT_TMP;
/**
 * FUNCTION_PURPOSE: Serial interrupt, echo received data.
 * FUNCTION_INPUTS : P3.0(RXD) serial input
 * FUNCTION_OUTPUTS: P3.1(TXD) serial output
 */

void Serial_1_ISR (void) interrupt 15 
{
    if (RI_1 == 1)
    {
        if (Receive_Get == 0)                           // If Receive_Buffer Not Full.
        {
            Receive_Buffer[u8Receive_Buffer_Index] = SBUF_1;
            u8Receive_Buffer_Index++;
            
            if (u8Receive_Buffer_Index == Receive_Buffer_Size)
            {
                Receive_Get = 1;
            }
        }
        
        clr_RI_1;                                         // Clear RI (Receive Interrupt).
    }
    if (TI_1 == 1)
    {
        Transmit_Busy = 0;                              // Mark Transmit Not Busy.
        clr_TI_1;                                         // Clear TI (Transmit Interrupt).
    }      
}

/************************************************************************************************************
*    Main function 
************************************************************************************************************/
void main (void)
{
    /* Note
       MCU power on system clock is HIRC (11.0592MHz), so Fsys = 11.0592MHz
    */
    
    Set_All_GPIO_Quasi_Mode();
    
    /* Change system closk source */
    #if SYS_CLK_EN == 1
        #if   SYS_SEL == 0
            System_Clock_Select(E_HXTEN);   //Fosc = 2~16MHz XTAL
        #elif SYS_SEL == 1
            System_Clock_Select(E_LXTEN);   //Fosc = 32.768KHz XTAL
        #elif SYS_SEL == 2
            System_Clock_Select(E_HIRCEN);  //Fosc = 11.0592MHz Internal RC
        #elif SYS_SEL == 3
            System_Clock_Select(E_LIRCEN);  //Fosc = 10KHz Internal RC
        #elif SYS_SEL == 4
            System_Clock_Select(E_OSCEN);   //Fosc = OSC-In External OSC
        #endif
    #endif
    
    #if SYS_DIV_EN == 1
        CKDIV = SYS_DIV;                                //Fsys = Fosc / (2* CLKDIV) = Fcpu
    #endif
    
    SCON_1 = 0x92;                                      //9600 bps @614.4kHz

#if 0
    while(1)
        Send_Data_To_UART1(0x55);
#else
    {
        UINT8 i;
        
        IE |= 0x80;                                     //EA=1
        EIE1 |= 0x01;                                   //ES_1=1
        while(Receive_Get == 0);                        //waiting the PC data
        
        clr_EA;                                         //disable interrupt
        for(i=0;i<Receive_Buffer_Size;i++)
        {
            P2 = Receive_Buffer[i];                     //show receive data to Port2
            Send_Data_To_UART1(Receive_Buffer[i]);         //show receive data to PC
        }
        while(1);
    }
#endif
}

